from ecsclient.common.user_management import authentication_provider, secret_key, \
    password_group, management_user, object_user

authentication_provider = authentication_provider
secret_key = secret_key
password_group = password_group
management_user = management_user
object_user = object_user
